/**
 * This file is resposible for running the starting text function and creating the package.json file
 * The file is also responsible to return the user answers to be used in questions after it
 */
const _ = require('lodash')
const fs = require('fs-extra')
const path = require('path')
const inquirer = require('inquirer')
const gitUserName = require('git-user-name')
const semver = require('semver')
const isValidNpmName = require('is-valid-npm-name')
const spdx = require('spdx')
const startingText = require('./starting-text')

const questions = [
  {
    type: 'input',
    name: 'name',
    message: 'Name: ',
    default: _.replace(_.lowerCase(path.basename(process.cwd())), /\s/g, ''),
    validate: (input) => {
      const valid = isValidNpmName(input)
      if (valid !== true) {
        return valid
      }
      return true
    }
  },
  {
    type: 'input',
    name: 'version',
    message: 'Version: ',
    default: '0.0.0',
    validate: (input) => {
      const valid = semver.valid(input)
      if (valid === null) {
        return 'Invalid version, please try again'
      }
      return true
    }
  },
  { type: 'input', name: 'description', message: 'Description: ' },
  { type: 'input', name: 'main', message: 'Entry Point: ', default: 'app.js' },
  { type: 'input', name: 'repository', message: 'Repository: ' },
  { type: 'input', name: 'keywords', message: 'Keywords: ' },
  { type: 'input', name: 'test', message: 'Test command: ' },
  { type: 'input', name: 'start', message: 'Start command: ' },
  { type: 'input', name: 'author', message: 'Author: ', default: gitUserName() },
  {
    type: 'input',
    name: 'license',
    message: 'License: ',
    default: 'MIT',
    validate: (input) => {
      const valid = spdx.valid(input)
      if (valid !== true) {
        return 'Invalid SPDX license, please try again'
      }
      return true
    }
  },
  { type: 'confirm', name: 'privatePackage', message: 'Private ?', default: false },
  { type: 'confirm', name: 'preferGlobal', message: 'Global ?', default: false }
]

module.exports = () => new Promise((resolve, reject) => {
  startingText()
  inquirer.prompt(questions).then((answers) => {
    let { name, version, description, main, repository, keywords, test, start, author, license, privatePackage, preferGlobal } = answers

    if (_.isUndefined(name) || _.isUndefined(name)) {
      reject(new Error('Could not get initial properties for the package.json file'))
    } else {
      /* The initial package.json model */
      let obj = {
        name,
        version,
        description,
        main,
        repository,
        keywords: _.split(keywords, /\s/g),
        scripts: {
          test,
          start
        },
        author,
        license,
        private: privatePackage,
        preferGlobal
      }

      /* Deleting all empty properties after the user has answered the questions to write the smallest package.json file possible */
      if (_.isEmpty(description)) {
        delete obj.description
      }
      if (_.isEmpty(repository)) {
        delete obj.repository
      }
      if (_.isEmpty(keywords)) {
        delete obj.keywords
      }
      if (_.isEmpty(test)) {
        delete obj.scripts.test
      }
      if (_.isEmpty(start)) {
        delete obj.scripts.start
      }
      if (_.isEmpty(start) && _.isEmpty(test)) {
        delete obj.scripts
      }
      if (_.isEmpty(author)) {
        delete obj.author
      }
      if (privatePackage === false) {
        delete obj.private
      }
      if (preferGlobal === false) {
        delete obj.preferGlobal
      }

      fs.outputFileSync(`${process.cwd()}/package.json`, JSON.stringify(obj, null, '\t'))

      resolve(obj)
    }
  })
})
